@@ -15,8 +15,9 @@
 from __future__ import annotations
 
 import inspect
-from typing import Any, Callable, Iterable, Mapping, Optional
+from typing import Any, Callable, Iterable, Mapping
 
+import tabulate
 from public import private, public
 from toolz import curry
 
@@ -46,7 +47,6 @@ from .core import (
     CrossJoin,
     Difference,
     DifferenceAll,
-    FullProjector,
     GroupBy,
     InnerJoin,
     Intersect,
@@ -59,12 +59,12 @@ from .core import (
     PartitionBy,
     Predicate,
     Projection,
+    Projector,
     Relation,
     RightJoin,
     Selection,
     SortBy,
     Table,
-    Tuple,
     Union,
     UnionAll,
 )
@@ -220,9 +220,16 @@ def right_join(right: Relation, predicate: JoinPredicate, left: Relation) -> Rig
     return RightJoin(left, right, predicate)
 
 
+@public  # type: ignore[misc]
+@shiftable
+def full_join(right: Relation, predicate: JoinPredicate, left: Relation) -> Relation:
+    """Full outer join."""
+    raise NotImplementedError("full outer joins are not yet supported")
+
+
 @private  # type: ignore[misc]
 @shiftable
-def _order_by(order_by: Tuple[OrderBy, ...], nulls: Nulls, child: Relation) -> SortBy:
+def _order_by(order_by: tuple[OrderBy, ...], nulls: Nulls, child: Relation) -> SortBy:
     return SortBy(child, order_by, nulls)
 
 
@@ -261,12 +268,14 @@ def order_by(*order_by: OrderBy, nulls: Nulls = Nulls.FIRST) -> SortBy:
 
 @private  # type: ignore[misc]
 @shiftable
-def _select(projectors: Mapping[str, FullProjector], child: Relation) -> Projection:
+def _select(
+    projectors: Mapping[str, Projector | WindowAggregateSpecification], child: Relation
+) -> Projection:
     return Projection(child, projectors)
 
 
 @public  # type: ignore[misc]
-def select(**projectors: FullProjector) -> Projection:
+def select(**projectors: Projector | WindowAggregateSpecification) -> Projection:
     """Subset or compute new columns from `projectors`.
 
     Parameters
@@ -305,12 +314,14 @@ def select(**projectors: FullProjector) -> Projection:
 
 @private  # type: ignore[misc]
 @shiftable
-def _mutate(mutators: Mapping[str, FullProjector], child: Relation) -> Mutate:
+def _mutate(
+    mutators: Mapping[str, Projector | WindowAggregateSpecification], child: Relation
+) -> Mutate:
     return Mutate(child, mutators)
 
 
 @public  # type: ignore[misc]
-def mutate(**mutators: FullProjector) -> Mutate:
+def mutate(**mutators: Projector | WindowAggregateSpecification) -> Mutate:
     """Add new columns specified by `mutators`.
 
     Parameters
@@ -681,7 +692,7 @@ def limit(limit: int, relation: Relation, *, offset: int = 0) -> Limit:
 
 
 @public  # type: ignore[misc]
-def count(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
+def count(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
     """Count the number of non-NULL values of `x`.
 
     Parameters
@@ -720,7 +731,7 @@ def total(x: Callable[[AbstractRow], R]) -> AggregateSpecification:
 
 
 @public  # type: ignore[misc]
-def first(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
+def first(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
     """Compute the first row of `x` over a window.
 
     Parameters
@@ -733,7 +744,7 @@ def first(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
 
 
 @public  # type: ignore[misc]
-def last(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
+def last(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
     """Compute the last row of `x` over a window.
 
     Parameters
@@ -747,8 +758,8 @@ def last(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
 
 @public  # type: ignore[misc]
 def nth(
-    x: Callable[[AbstractRow], Optional[T]],
-    i: Callable[[AbstractRow], Optional[int]],
+    x: Callable[[AbstractRow], T | None],
+    i: Callable[[AbstractRow], int | None],
 ) -> AggregateSpecification:
     """Compute the `i`-th row of `x` over a window.
 
@@ -783,9 +794,9 @@ def dense_rank() -> AggregateSpecification:
 
 @public  # type: ignore[misc]
 def lead(
-    x: Callable[[AbstractRow], Optional[T]],
-    n: Callable[[AbstractRow], Optional[int]] = (lambda row: 1),
-    default: Callable[[AbstractRow], Optional[T]] = (lambda row: None),
+    x: Callable[[AbstractRow], T | None],
+    n: Callable[[AbstractRow], int | None] = (lambda row: 1),
+    default: Callable[[AbstractRow], T | None] = (lambda row: None),
 ) -> AggregateSpecification:
     """Lead a column `x` by `n` rows, using `default` for NULL values.
 
@@ -808,9 +819,9 @@ def lead(
 
 @public  # type: ignore[misc]
 def lag(
-    x: Callable[[AbstractRow], Optional[T]],
-    n: Callable[[AbstractRow], Optional[int]] = (lambda row: 1),
-    default: Callable[[AbstractRow], Optional[T]] = (lambda row: None),
+    x: Callable[[AbstractRow], T | None],
+    n: Callable[[AbstractRow], int | None] = (lambda row: 1),
+    default: Callable[[AbstractRow], T | None] = (lambda row: None),
 ) -> AggregateSpecification:
     """Lag a column `x` by `n` rows, using `default` for NULL values.
 
@@ -845,7 +856,7 @@ def mean(x: Callable[[AbstractRow], R]) -> AggregateSpecification:
 
 
 @public  # type: ignore[misc]
-def min(x: Callable[[AbstractRow], Optional[Comparable]]) -> AggregateSpecification:
+def min(x: Callable[[AbstractRow], Comparable | None]) -> AggregateSpecification:
     """Compute the minimum of a column.
 
     Parameters
@@ -858,7 +869,7 @@ def min(x: Callable[[AbstractRow], Optional[Comparable]]) -> AggregateSpecificat
 
 
 @public  # type: ignore[misc]
-def max(x: Callable[[AbstractRow], Optional[Comparable]]) -> AggregateSpecification:
+def max(x: Callable[[AbstractRow], Comparable | None]) -> AggregateSpecification:
     """Compute the maximum of a column.
 
     Parameters
@@ -954,3 +965,67 @@ def stdev_pop(x: Callable[[AbstractRow], R]) -> AggregateSpecification:
 
     """
     return AggregateSpecification(PopulationStandardDeviation, (x,))
+
+
+@public  # type: ignore[misc]
+@shiftable
+def pretty(
+    rows: Relation,
+    *,
+    n: int | None = 10,
+    tablefmt: str = "simple",
+    headers: str = "keys",
+    **kwargs: Any,
+) -> str:
+    """Pretty-format a relation.
+
+    Parameters
+    ----------
+    rows
+        The relation to print
+    n
+        The number of rows to format; A value of `None` will pretty format all
+        rows
+    tablefmt
+        The kind of table to use for formatting
+    headers
+        A string indicating how to compute column names
+    kwargs
+        Additional keyword arguments passed to the `tabulate.tabulate`
+        function
+
+    Returns
+    -------
+    str
+        Pretty-formatted relation
+
+    See Also
+    --------
+    stupidb.api.show
+    """
+    return tabulate.tabulate(
+        (rows >> limit(n)) if n is not None else rows,
+        tablefmt=tablefmt,
+        headers=headers,
+        **kwargs,
+    )
+
+
+@public  # type: ignore[misc]
+@shiftable
+def show(rows: Relation, **kwargs: Any) -> None:
+    """Pretty-print a relation.
+
+    Parameters
+    ----------
+    rows
+        The relation to print
+    kwargs
+        Additional keyword arguments passed to the `stupidb.api.pretty`
+        function
+
+    See Also
+    --------
+    stupidb.api.pretty
+    """
+    print(pretty(rows, **kwargs))
