@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """Tests for `stupidb` package."""
 
 import builtins
@@ -30,6 +28,7 @@ from stupidb.api import (
     cov_samp,
     cross_join,
     exists,
+    full_join,
     group_by,
     inner_join,
     left_join,
@@ -40,6 +39,7 @@ from stupidb.api import (
     mutate,
     order_by,
     over,
+    pretty,
     right_join,
     select,
     sift,
@@ -218,6 +218,58 @@ ON t.z = s.z"""
     assert_rowset_equal(result, expected)
 
 
+@pytest.mark.xfail(
+    raises=NotImplementedError,
+    reason="full outer joins are not yet supported",
+)
+def test_full_join(employee, department, con):
+    join = (
+        table(employee)
+        >> full_join(
+            table(department), lambda e, d: e["department_id"] == d["department_id"]
+        )
+        >> select(
+            last_name=lambda row: row.left["last_name"],
+            e_dep_id=lambda row: row.left["department_id"],
+            department_name=lambda row: row.right["department_name"],
+            d_dep_id=lambda row: row.right["department_id"],
+        )
+    )
+    result = list(join)
+    query = """
+with
+left_outer as (
+    SELECT
+        e.last_name,
+        e.department_id as e_dep_id,
+        d.department_name,
+        d.department_id AS d_dep_id
+    FROM employee e
+    LEFT OUTER JOIN department d
+    USING (department_id)
+),
+right_outer as (
+    SELECT
+        e.last_name,
+        e.department_id as e_dep_id,
+        d.department_name,
+        d.department_id AS d_dep_id
+    FROM department d
+    LEFT OUTER JOIN employee e
+    USING (department_id)
+)
+SELECT * FROM left_outer
+UNION
+SELECT * FROM right_outer
+"""
+    expected = [
+        dict(zip(("last_name", "e_dep_id", "department_name", "d_dep_id"), row))
+        for row in con.execute(query).fetchall()
+    ]
+    breakpoint()
+    assert_rowset_equal(result, expected)
+
+
 def test_semi_join():
     rows = [
         dict(z="a", a=1, b=2),
@@ -641,3 +693,51 @@ def test_variance_window(t_rows):
         ),
     }
     assert set(result) == expected
+
+
+def test_pretty(t_rows):
+    window = Window.range(partition_by=[lambda r: r.name])
+    query = table(t_rows) >> select(
+        min_date=min(lambda r: r.date) >> over(window),
+        max_date=max(lambda r: r.date) >> over(window),
+    )
+    expected = """\
+min_date    max_date
+----------  ----------
+2018-01-01  2018-01-07
+2018-01-01  2018-01-07
+2018-01-01  2018-01-07
+2018-01-01  2018-01-07
+2018-01-02  2018-01-04
+2018-01-02  2018-01-04
+2018-01-02  2018-01-04"""
+    result = query >> pretty()
+    assert result == expected
+
+
+def test_pretty_fmt(t_rows):
+    window = Window.range(partition_by=[lambda r: r.name])
+    query = table(t_rows) >> select(
+        min_date=min(lambda r: r.date) >> over(window),
+        max_date=max(lambda r: r.date) >> over(window),
+    )
+    expected = """\
+╒════════════╤════════════╕
+│ min_date   │ max_date   │
+╞════════════╪════════════╡
+│ 2018-01-01 │ 2018-01-07 │
+├────────────┼────────────┤
+│ 2018-01-01 │ 2018-01-07 │
+├────────────┼────────────┤
+│ 2018-01-01 │ 2018-01-07 │
+├────────────┼────────────┤
+│ 2018-01-01 │ 2018-01-07 │
+├────────────┼────────────┤
+│ 2018-01-02 │ 2018-01-04 │
+├────────────┼────────────┤
+│ 2018-01-02 │ 2018-01-04 │
+├────────────┼────────────┤
+│ 2018-01-02 │ 2018-01-04 │
+╘════════════╧════════════╛"""
+    result = query >> pretty(tablefmt="fancy_grid")
+    assert result == expected
